"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ServerPlugin = void 0;

var _tapable = require("tapable");

var _type = require("@spark-build/web-utils/lib/type");

var _Api = require("./Api");

var _util = require("./util");

var _constants = require("../../constants");

class ServerPlugin {
  constructor(server) {
    this.plugins = new Map();
    this.skipIds = new Set();
    this.initPluginReady = false;
    this.server = server;
  }

  get isReady() {
    return this.initPluginReady;
  }

  getPlugins() {
    return this.plugins;
  }

  register(id, plugin) {
    this.plugins.set(id, plugin);
    return this;
  }

  skip(id) {
    this.skipIds.add(id);
  }

  removeSkip(id) {
    if (this.skipIds.has(id)) {
      this.skipIds.delete(id);
    }
  }

  enableStatus(id) {
    if (!this.plugins.has(id) || this.skipIds.has(id)) {
      return false;
    }

    if (this.server.userConfig[id] === false) {
      return false;
    }

    return true;
  }

  getPluginAPI(id) {
    const plugin = new _Api.PluginApi(id, this.server);
    return new Proxy(plugin, {
      get: (target, prop) => {
        if (this.server.methods.has(prop)) {
          return this.server.methods.get(prop);
        }

        const findProp = (t = target) => (0, _type.isFn)(t[prop]) ? t[prop].bind(t) : t[prop];

        if (this[prop]) {
          return findProp(this);
        }

        if (this.server[prop]) {
          return findProp(this.server);
        }

        return findProp();
      }
    });
  }

  async initPlugins() {
    const plugins = await (0, _util.loadPlugins)();
    const tInitPlugin = new _tapable.AsyncSeriesWaterfallHook(['initPlugin']);

    for (const plugin of plugins) {
      tInitPlugin.tapPromise({
        name: plugin.id,
        stage: plugin.stage
      }, async () => {
        const pluginInstance = this.getPluginAPI(plugin.id);
        await plugin.fn(pluginInstance);
        return this.register(plugin.id, pluginInstance);
      });
    }

    await tInitPlugin.promise(undefined);
    this.initPluginReady = true;
    this.server.applyPlugins({
      key: 'onPluginReady',
      type: _constants.ApplyPluginsType.EVENT
    });
    return this;
  }

}

exports.ServerPlugin = ServerPlugin;